import os

import airflow
from airflow.utils import operator_helpers
from dbnd_airflow.airflow_override.operator_helpers import context_to_airflow_vars

__patches = (
  #   (airflow.models, "TaskInstance", DbndAirflowTaskInstance),
  #   (airflow.models.taskinstance, "TaskInstance", DbndAirflowTaskInstance),
    (
        airflow.utils.operator_helpers,
        "context_to_airflow_vars",
        context_to_airflow_vars,
    ),
)


def patch_module_attr(module, name, value):
    original_name = "original_" + name
    if getattr(module, original_name, None):
        return
    setattr(module, original_name, getattr(module, name))
    setattr(module, name, value)


def unpatch_module_attr(module, name, value):
    setattr(module, name, getattr(module, "original_" + name))


def patch_models():
    for module, name, value in __patches:
        patch_module_attr(module, name, value)


def unpatch_models():
    for module, name, value in __patches:
        unpatch_module_attr(module, name, value)


def monkeypatch_airflow():
    patch_models()



#!/usr/bin/env python
# PYTHON_ARGCOMPLETE_OK
# -*- coding: utf-8 -*-
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
import logging
import os
import subprocess

import argcomplete


# DO NOT IMPORT ANYTHING FROM AIRFLOW
# we need to initialize some config values first


def subprocess_airflow(args):
    """Forward arguments to airflow command line"""
    args = ["airflow"] + args
    logging.info("Running airflow command: %s", subprocess.list2cmdline(args))
    subprocess.check_call(args=args)

    logging.info("Airflow command has been successfully executed")


def main(args=None):
    # from dbnd._core.log.config import configure_basic_logging
    # configure_basic_logging(None)

    from dbnd import dbnd_config
    from dbnd._core.configuration.environ_config import set_quiet_mode
    from dbnd._core.context.bootstrap import dbnd_system_bootstrap

    set_quiet_mode()
    dbnd_system_bootstrap()

    # LET'S PATCH AIRFLOW FIRST
    from dbnd_airflow.bootstrap import airflow_bootstrap

    airflow_bootstrap()

    from airflow.bin.cli import CLIFactory
    from airflow.configuration import conf
    from dbnd_airflow.plugins.setup_plugins import (
        setup_scheduled_dags,
        setup_versioned_dags,
    )

    # ORIGINAL CODE from  airflow/bin/airflow
    if conf.get("core", "security") == "kerberos":
        os.environ["KRB5CCNAME"] = conf.get("kerberos", "ccache")
        os.environ["KRB5_KTNAME"] = conf.get("kerberos", "keytab")

    parser = CLIFactory.get_parser()
    argcomplete.autocomplete(parser)
    args = parser.parse_args(args=args)
    func_name = args.func.__name__

    # DBND PATCH:
    if dbnd_config.getboolean("airflow", "auto_add_scheduled_dags") and func_name in [
        "scheduler",
        "webserver",
    ]:
        setup_scheduled_dags()
    if dbnd_config.getboolean("airflow", "auto_add_versioned_dags") and func_name in [
        "webserver"
    ]:
        setup_versioned_dags()

    args.func(args)


if __name__ == "__main__":
    main()



import logging
import os

from dbnd._core.configuration.environ_config import in_quiet_mode

logger = logging.getLogger(__name__)


# WE SHOULD NOT HAVE ANY dbnd imports here -- circle import:  dbnd_airflow -> airflow -> load .. -> import dbnd_airflow

def use_databand_airflow_dagbag():
    """
    Overriding Airflow Dagbag, so versioned dags can be used
    :return:
    """

    import airflow
    from airflow import settings
    if settings.RBAC:
        from airflow.www_rbac import views
    else:
        from airflow.www import views
    from dbnd_airflow.airflow_override import patch_module_attr

    logging.info("Using dbnd dagbag (supporting versioned dags).")
    from dbnd_airflow.web.databand_versioned_dagbag import DbndAirflowDagBag, DbndDagModel

    if os.environ.get("SKIP_DAGS_PARSING") != "True":
        views.dagbag = DbndAirflowDagBag(settings.DAGS_FOLDER)
    else:
        views.dagbag = DbndAirflowDagBag(os.devnull, include_examples=False)

    # some views takes dag from dag model
    if hasattr(views, "DagModel"):
        patch_module_attr(views, "DagModel", DbndDagModel)

    # dag_details invoke DagBag directly
    patch_module_attr(airflow.models, "DagBag", DbndAirflowDagBag)
    patch_module_attr(airflow.models.dag, "DagBag", DbndAirflowDagBag)
    patch_module_attr(airflow.models.dag, "DagModel", DbndDagModel)


def patch_airflow_create_app():
    if not in_quiet_mode():
        logger.debug("Adding support for versioned DBND DagBag")

    def patch_create_app(create_app_func):
        def patched_create_app(*args, **kwargs):
            from dbnd._core.configuration.dbnd_config import config
            from dbnd_airflow._plugin import configure_airflow_sql_alchemy_conn
            from dbnd_airflow.bootstrap import _register_sqlachemy_local_dag_job

            _register_sqlachemy_local_dag_job()

            logger.info("Setting SQL from databand configuration.")
            config.load_system_configs()
            configure_airflow_sql_alchemy_conn()

            res = create_app_func(*args, **kwargs)
            use_databand_airflow_dagbag()
            return res

        return patched_create_app

    from airflow.www_rbac import app as airflow_app_rbac
    airflow_app_rbac.create_app = patch_create_app(airflow_app_rbac.create_app)

    from airflow.www import app as airflow_app_no_rbac
    airflow_app_no_rbac.create_app = patch_create_app(airflow_app_no_rbac.create_app)



from datetime import datetime, timedelta

from dbnd import parameter
from dbnd._core.task import Config
from dbnd._core.utils.platform.windows_compatible.getuser import dbnd_getuser


class AirflowConfig(Config):
    """Apache Airflow (currently a part of Databand)"""

    _conf__task_family = "airflow"

    optimize_airflow_db_access = parameter(
        description="Enable all Airflow DB access optimizations"
    )[bool]

    # enabled by optimize_airflow_db_access
    disable_db_ping_on_connect = parameter(
        description="Optimize DB access performance by disabling starndard airflow ping "
        "on every sqlalchmey connectiion initializaiton"
    )[bool]

    disable_dag_concurrency_rules = parameter(
        description="Optimize DB access performance by disabling"
        " dag and task concurrency checks"
    )[bool]

    dbnd_dag_concurrency = parameter(description="Concurrency for dbnd ad-hoc dags")[
        int
    ]

    remove_airflow_std_redirect = parameter(
        default=False,
        description="Remove airflow stdout/stderr redirection into logger on DBND operator run "
        "(redirect can cause crash due to loopback between streams",
    )[bool]

    sql_alchemy_conn = parameter(
        description="key used by airflow to encrypt connections credentials "
        "(use 'dbnd' for using value from `core.fernet_key`)",
        default=None,
    )[str]
    fernet_key = parameter(
        description="key used by airflow to encrypt connections credentials "
        "(use 'dbnd' for using value from `core.fernet_key`)",
        default=None,
    )[str]
    auto_add_versioned_dags = parameter(
        default=True,
        description="Auto add versioned dag support to dbnd-airflow command",
    )
    auto_add_scheduled_dags = parameter(
        default=True,
        description="Auto add dbnd scheduled dags to airflow dags on dbnd-airflow command ",
    )
    auto_disable_scheduled_dags_load = parameter(
        default=True,
        description="Auto disable dbnd scheduled dags load in databand clie",
    )
    webserver_url = parameter(
        default=None, description="URL of airflow webserver used by local runs"
    )[str]

    def __init__(self, *args, **kwargs):
        super(AirflowConfig, self).__init__(*args, **kwargs)
        if not self.optimize_airflow_db_access:
            self.disable_db_ping_on_connect = False
            self.disable_dag_concurrency_rules = False


username = dbnd_getuser()
default_args = {
    "owner": username,
    "depends_on_past": False,
    "start_date": datetime(2015, 6, 1),
    "email": ["airflow@example.com"],
    "email_on_failure": False,
    "email_on_retry": False,
    "retries": 0,
    "retry_delay": timedelta(minutes=5),
    # 'queue': 'bash_queue',
    # 'pool': 'backfill',
    # 'priority_weight': 10,
    # 'end_date': datetime(2016, 1, 1),
}


def get_dbnd_default_args():
    return default_args.copy()



# -*- coding: utf-8 -*-
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

import logging
import os
import sys

from copy import deepcopy

from airflow import configuration as conf
from airflow.config_templates.airflow_local_settings import (
    DEFAULT_LOGGING_CONFIG as airflow_default_log_config,
)
from airflow.utils.log.file_processor_handler import FileProcessorHandler


windows_compatible_mode = os.name == "nt"


class FileProcessorHandlerWinCompatible(FileProcessorHandler):
    """
    FileProcessorHandler is a python log handler that handles
    dag processor logs. It creates and delegates log handling
    to `logging.FileHandler` after receiving dag processor context.
    """

    def set_context(self, filename):
        """
        Provide filename context to airflow task handler.
        :param filename: filename in which the dag is located
        """
        super(FileProcessorHandlerWinCompatible, self).set_context(filename)

        logging.debug(
            "Processor logging is up at %s for %s", self.handler.baseFilename, filename
        )

    def _symlink_latest_log_directory(self):
        """
        Create symbolic link to the current day's log directory to
        allow easy access to the latest scheduler log files.

        :return: None
        """

        if windows_compatible_mode:
            return
        super(FileProcessorHandlerWinCompatible, self)._symlink_latest_log_directory()


class FileProcessorErrorHandler(logging.StreamHandler):
    def __init__(self, **kwargs):
        super(FileProcessorErrorHandler, self).__init__(**kwargs)

    def emit(self, record):
        if "ERROR" in record.msg:
            super(FileProcessorErrorHandler, self).emit(record)


PROCESSOR_LOG_FOLDER = conf.get("scheduler", "CHILD_PROCESS_LOG_DIRECTORY")

# this config will be used automatically by airflow,
# but we will override it the first time we can (right after import)
DEFAULT_LOGGING_CONFIG = deepcopy(airflow_default_log_config)
# we need to replace the original with this one for windows support
wp = "%s.%s" % (__name__, FileProcessorHandlerWinCompatible.__name__)
error_handler = "%s.%s" % (__name__, FileProcessorErrorHandler.__name__)

DEFAULT_LOGGING_CONFIG["handlers"]["processor"]["class"] = wp

# the default airflow "console" handler somehow ends up redirecting back to processor logger, getting stuck in an infinite loop
# sys.stdout at this point is Raw stdout without redirection, as this code runs at airflow boot time
# ( no airflow task actually is running, so there is no stdout_redirect applied )
DEFAULT_LOGGING_CONFIG["handlers"]["stdout"] = {
    "class": error_handler,
    "formatter": "airflow",
    "stream": sys.stdout,
    "level": "INFO",
}

DEFAULT_LOGGING_CONFIG["loggers"]["airflow.processor"]["handlers"].append("stdout")



import logging
import os

from sqlalchemy.event import Events


logger = logging.getLogger(__name__)


def reinit_airflow_sql_conn():
    from airflow.settings import configure_vars, configure_orm
    from dbnd._core.configuration.dbnd_config import config as dbnd_config

    configure_vars()
    # The webservers import this file from models.py with the default settings.
    configure_orm()
    # add query handler before every execute
    # this will print query, code line and stack trace
    if dbnd_config.getboolean("log", "sqlalchemy_trace"):
        from sqlalchemy import event
        from airflow import settings as airflow_settings
        from dbnd_airflow.db_utils import trace_sqlalchemy_query

        event.listen(
            airflow_settings.engine, "before_cursor_execute", trace_sqlalchemy_query
        )

    # this will print query execution time
    if dbnd_config.getboolean("log", "sqlalchemy_profile"):
        from sqlalchemy import event
        from airflow import settings as airflow_settings
        from dbnd_airflow.db_utils import (
            profile_before_cursor_execute,
            profile_after_cursor_execute,
        )

        event.listen(
            airflow_settings.engine,
            "before_cursor_execute",
            profile_before_cursor_execute,
        )
        event.listen(
            airflow_settings.engine,
            "after_cursor_execute",
            profile_after_cursor_execute,
        )



import logging
import os
import sys


def _register_sqlachemy_local_dag_job():
    from dbnd_airflow.scheduler.single_dag_run_job import SingleDagRunJob

    # we also need to load this object, so webserver will see our LocalDagJobs objectsimpo
    return SingleDagRunJob


def _fix_sys_path():
    # airflow append DAG and PLUGIN folders to sys.path
    # they are coming from ConfigParser, python2 backport create newbytes values instead of str
    # any module that uses Path will fail on that
    sys.path = [str(p) if type(p) != str else p for p in sys.path]


def set_airflow_sql_conn_from_dbnd_config():
    logging.debug("updating airflow config from dbnd config")
    from dbnd._core.configuration.dbnd_config import config as dbnd_config

    sql_alchemy_conn = dbnd_config.get("airflow", "sql_alchemy_conn")
    if sql_alchemy_conn == "dbnd":
        logging.debug("updating airflow sql from dbnd core.sql_alchemy_conn")
        sql_alchemy_conn = dbnd_config.get("core", "sql_alchemy_conn")

    if sql_alchemy_conn and "AIRFLOW__CORE__SQL_ALCHEMY_CONN" not in os.environ:
        os.environ["AIRFLOW__CORE__SQL_ALCHEMY_CONN"] = sql_alchemy_conn

    fernet_key = dbnd_config.get("airflow", "fernet_key")
    if fernet_key == "dbnd":
        fernet_key = dbnd_config.get("core", "fernet_key")
    if fernet_key and "AIRFLOW__CORE__FERNET_KEY" not in os.environ:
        os.environ["AIRFLOW__CORE__FERNET_KEY"] = fernet_key


_airflow_bootstrap_applied = False


def airflow_bootstrap():
    global _airflow_bootstrap_applied
    if _airflow_bootstrap_applied:
        return
    set_airflow_sql_conn_from_dbnd_config()

    from dbnd_airflow.airflow_override import monkeypatch_airflow

    monkeypatch_airflow()
    from dbnd_airflow_windows.airflow_windows_support import (
        enable_airflow_windows_support,
    )

    if os.name == "nt":
        enable_airflow_windows_support()
    _fix_sys_path()
    _register_sqlachemy_local_dag_job()
    _airflow_bootstrap_applied = True



import getpass
import os
import sys

from airflow.utils import configuration, timeout

from dbnd_airflow_windows import timeout as timeout_patch
from dbnd_airflow_windows.configuration import tmp_configuration_copy
from dbnd_airflow_windows.getuser import find_runnable_getuser_function


__win_patches = (
    (timeout, "timeout", timeout_patch.timeout),
    (configuration, "tmp_configuration_copy", tmp_configuration_copy),
    (getpass, "getuser", find_runnable_getuser_function()),
    (sys, "maxint", sys.maxsize),
)


def patch_module_attr(module, name, value):
    original_name = "original_" + name
    if hasattr(module, original_name):
        return
    setattr(module, original_name, getattr(module, name, None))
    setattr(module, name, value)


def _patch_windows_compatible():
    for module, name, value in __win_patches:
        patch_module_attr(module, name, value)


def enable_airflow_windows_support():
    if os.name == "nt":
        _patch_windows_compatible()



import logging
import os
import typing

import dbnd


AIRFLOW_LEGACY_URL_KEY = "airflow"
logger = logging.getLogger(__name__)

if typing.TYPE_CHECKING:
    from typing import Optional

    from dbnd._core.context.databand_context import DatabandContext
    from dbnd._core.task_run.task_run import TaskRun


@dbnd.hookimpl
def dbnd_setup_plugin():
    # Set additional airflow configuration
    configure_airflow_sql_alchemy_conn()

    from dbnd import register_config_cls
    from dbnd_airflow.config import AirflowConfig

    register_config_cls(AirflowConfig)


@dbnd.hookimpl
def dbnd_on_exit_context(ctx):
    if ctx._airflow_op_catcher_dag:
        ctx._airflow_op_catcher_dag.__exit__(None, None, None)


@dbnd.hookimpl
def dbnd_post_enter_context(ctx):  # type: (DatabandContext) -> None
    from dbnd_airflow.dbnd_task_executor.airflow_operators_catcher import (
        DatabandOpCatcherDag,
    )

    from dbnd_airflow.config import get_dbnd_default_args

    ctx._airflow_op_catcher_dag = DatabandOpCatcherDag(
        dag_id="inline_airflow_ops", default_args=get_dbnd_default_args()
    )
    ctx._airflow_op_catcher_dag.__enter__()


def configure_airflow_sql_alchemy_conn():
    from dbnd_airflow.airflow_extensions.airflow_config import reinit_airflow_sql_conn

    reinit_airflow_sql_conn()


@dbnd.hookimpl
def dbnd_setup_unittest():
    os.environ["AIRFLOW__CORE__UNIT_TEST_MODE"] = "True"
    from airflow import configuration as airflow_configuration
    from airflow.configuration import TEST_CONFIG_FILE

    # we can't call load_test_config, as it override airflow.cfg
    # we want to keep it as base
    logger.info("Reading Airflow test config at %s" % TEST_CONFIG_FILE)
    airflow_configuration.conf.read(TEST_CONFIG_FILE)

    sql_alchemy_conn = airflow_configuration.get("core", "sql_alchemy_conn")
    if sql_alchemy_conn.find("unittests.db") == -1:
        logger.warning(
            "You should set SQL_ALCHEMY_CONN to sqlite:///.../unittests.db for tests! %s"
            % sql_alchemy_conn
        )
    from dbnd_airflow.airflow_extensions.airflow_config import reinit_airflow_sql_conn

    reinit_airflow_sql_conn()

    from dbnd_airflow.dbnd_airflow_main import subprocess_airflow

    subprocess_airflow(args=["initdb"])



import logging

from airflow.hooks.base_hook import BaseHook


logger = logging.getLogger(__name__)


class AzureBlobStorageCredentials(BaseHook):
    def __init__(self, conn_id="azure_blob_storage_default"):
        self.conn_id = conn_id

    def get_credentials(self):
        connection_object = self.get_connection(self.conn_id)
        extras = connection_object.extra_dejson
        credentials = dict()

        if connection_object.login:
            credentials["account_name"] = connection_object.login
        if connection_object.password:
            credentials["account_key"] = connection_object.password
        credentials.update(extras)
        return credentials



from airflow import DAG


class DatabandOpCatcherDag(DAG):
    """
    we will use this Dag, when we have AirflowOperators in DatabandTasks
    for every operator `add_task` will be called, that will create DatabandTask
    so it can be registered and connected to graph
    """

    def add_task(self, task):
        super(DatabandOpCatcherDag, self).add_task(task)
        # task is AirflowOperator

        from dbnd_airflow.dbnd_task_executor.airflow_operator_as_dbnd import (
            AirflowOperatorAsDbndTask,
        )

        AirflowOperatorAsDbndTask.build_airflow_task(airflow_operator=task, dag=self)



from __future__ import print_function

import logging
import sys
import typing

from dbnd import dbnd_bootstrap
from dbnd_airflow.dbnd_task_executor.airflow_operator_as_dbnd import (
    AirflowOperatorAsDbndTask,
)
from dbnd_airflow_contrib.dbnd_operator import DbndOperator


if typing.TYPE_CHECKING:
    from dbnd._core.task_run.task_run import TaskRun

logger = logging.getLogger(__name__)


def dbnd_execute_airflow_operator(airflow_operator, context):
    """
    Airflow Operator execute function
    """
    dbnd_task_id = getattr(airflow_operator, "dbnd_task_id", None)
    if not dbnd_task_id:
        return airflow_operator.execute(context)

    # operator is wrapped/created by databand
    if isinstance(airflow_operator, DbndOperator):
        return airflow_operator.execute(context)

    from dbnd._core.current import get_databand_run

    # this is the Airflow native Operator
    # we will want to call it with Databand wrapper
    # we are at the airflow operator that is part of databand dag
    dbnd_task_run = get_databand_run().get_task_run_by_id(dbnd_task_id)
    if isinstance(dbnd_task_run.task, AirflowOperatorAsDbndTask):
        # we need to update it with latest, as we have "templated" and copy airflow operator object
        dbnd_task_run.task.airflow_op = airflow_operator
        return dbnd_task_run.runner.execute(context)
    else:
        logging.info(
            "Found airflow operator with dbnd_task_id that can not be run by dbnd: %s",
            airflow_operator,
        )
        return airflow_operator.execute(context)


# wrappers for DbndOperator


def _dbnd_operator_to_taskrun(operator):
    # type: (DbndOperator)-> TaskRun
    from dbnd._core.current import get_databand_run

    return get_databand_run().get_task_run_by_id(operator.dbnd_task_id)


def dbnd_operator__execute(dbnd_operator, context):
    from dbnd._core.current import try_get_databand_run
    from dbnd._core.run.databand_run import DatabandRun
    from targets import target

    run = try_get_databand_run()
    if not run:
        # we are not inside dbnd run, probably we are running from native airflow
        # let's try to load it:
        executor_config = dbnd_operator.executor_config
        logger.info("context: %s", context)

        logger.info("task.executor_config: %s", dbnd_operator.executor_config)
        logger.info("ti.executor_config: %s", context["ti"].executor_config)
        driver_dump = executor_config["DatabandExecutor"].get("dbnd_driver_dump")
        print(
            "Running dbnd task %s %s" % (dbnd_operator.dbnd_task_id, driver_dump),
            file=sys.__stderr__,
        )

        if executor_config["DatabandExecutor"].get(
            "remove_airflow_std_redirect", False
        ):
            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__

        dbnd_bootstrap()
        run = DatabandRun.load_run(
            dump_file=target(driver_dump), disable_tracking_api=False
        )

        with run.run_context() as dr:
            task_run = run.get_task_run_by_id(dbnd_operator.dbnd_task_id)
            ret_value = task_run.runner.execute(airflow_context=context)
    else:
        task_run = run.get_task_run_by_id(dbnd_operator.dbnd_task_id)
        ret_value = task_run.runner.execute(airflow_context=context)

    return ret_value


def dbnd_operator__kill(dbnd_operator):
    from dbnd._core.current import try_get_databand_run

    run = try_get_databand_run()
    if not run:
        return

    task_run = run.get_task_run_by_id(dbnd_operator.dbnd_task_id)
    return task_run.task.on_kill()



import airflow.utils.operator_helpers

from dbnd._core.current import try_get_databand_run


def context_to_airflow_vars(context, in_env_var_format=False):
    # original_context_to_airflow_vars is created during function override in patch_models()
    params = airflow.utils.operator_helpers.original_context_to_airflow_vars(
        context=context, in_env_var_format=in_env_var_format
    )
    if in_env_var_format:
        dbnd_run = try_get_databand_run()
        if dbnd_run:
            params.update(dbnd_run.get_context_spawn_env())
    return params



import logging
import os.path

from dbnd._core.utils.basics.format_exception import format_exception_as_str
from dbnd._core.utils.project.project_fs import abs_join, relative_path


logger = logging.getLogger(__name__)
_airflow_lib_home_default = relative_path(__file__)


def dbnd_airflow_path(*path):
    return abs_join(_airflow_lib_home_default, *path)


def link_dropin_file(source_path, target_path, unlink_first=True, name="file"):
    sub_dir = os.path.dirname(target_path)

    if unlink_first and os.path.islink(target_path):
        try:
            logger.info("unlinking existing %s at %s", name, target_path)
            os.unlink(target_path)
        except Exception:
            logger.error(
                "failed to unlink %s at %s: %s",
                (name, target_path, format_exception_as_str()),
            )
            return

    if not os.path.exists(target_path):
        try:
            logger.info("Linking %s to %s.", source_path, target_path)
            if not os.path.exists(sub_dir):
                os.makedirs(sub_dir)
            os.symlink(source_path, target_path)
        except Exception:
            logger.error(
                "failed to link %s in the airflow dags_folder: %s"
                % (name, format_exception_as_str())
            )



from os import path

import setuptools

from setuptools.config import read_configuration


BASE_PATH = path.dirname(__file__)
CFG_PATH = path.join(BASE_PATH, "setup.cfg")

config = read_configuration(CFG_PATH)
version = config["metadata"]["version"]

setuptools.setup(
    name="dbnd-airflow",
    package_dir={"": "src"},
    install_requires=[
        "dbnd==" + version,
        "apache-airflow==1.10.7",
        "Werkzeug<1.0.0",
        "future>=0.16.0, <0.17",
        "sqlalchemy_utc",
        "sqlalchemy_utils",
    ],
    extras_require=dict(
        tests=[
            # airflow support
            "pandas<1.0.0,>=0.17.1",
            # azure
            "azure-storage-blob",
            # aws
            "httplib2>=0.9.2",
            "boto3",
            "s3fs",
            # gcs
            "httplib2>=0.9.2",
            "google-api-python-client>=1.6.0, <2.0.0dev",
            "google-auth>=1.0.0, <2.0.0dev",
            "google-auth-httplib2>=0.0.1",
            "google-cloud-container>=0.1.1",
            "PyOpenSSL",
            "pandas-gbq",
            # docker
            "docker~=3.0",
            # k8s
            "kubernetes==9.0.0",
            "cryptography>=2.0.0",
        ]
    ),
    entry_points={
        "console_scripts": ["dbnd-airflow = dbnd_airflow.dbnd_airflow_main:main"],
        "dbnd": ["dbnd-airflow = dbnd_airflow._plugin"],
    },
)
